@@ -2,7 +2,7 @@
 /*                                                                        */
 /*  This file is part of Frama-C.                                         */
 /*                                                                        */
-/*  Copyright (C) 2007-2015                                               */
+/*  Copyright (C) 2007-2018                                               */
 /*    CEA (Commissariat à l'énergie atomique et aux énergies              */
 /*         alternatives)                                                  */
 /*                                                                        */
@@ -22,6 +22,14 @@
 
 #ifndef __FC_DEFINE_BLKCNT_T
 #define __FC_DEFINE_BLKCNT_T
-typedef unsigned int blkcnt_t; 
+#include "features.h"
+__PUSH_FC_STDLIB
+__BEGIN_DECLS
+#ifndef __blkcnt_t_defined
+typedef unsigned int blkcnt_t;
+#define __blkcnt_t_defined
+#endif
+__END_DECLS
+__POP_FC_STDLIB
 #endif
 
