@@ -2,7 +2,7 @@
 /*                                                                        */
 /*  This file is part of Frama-C.                                         */
 /*                                                                        */
-/*  Copyright (C) 2007-2015                                               */
+/*  Copyright (C) 2007-2018                                               */
 /*    CEA (Commissariat à l'énergie atomique et aux énergies              */
 /*         alternatives)                                                  */
 /*                                                                        */
@@ -22,6 +22,8 @@
 
 #ifndef __FC_DEFINE_STAT_H
 #define __FC_DEFINE_STAT_H
+#include "features.h"
+__PUSH_FC_STDLIB
 
 
 #include "__fc_define_ino_t.h"
@@ -36,6 +38,8 @@
 
 #define __statfs_word unsigned int
 
+__BEGIN_DECLS
+
 struct statfs {
 	__statfs_word f_type;
 	__statfs_word f_bsize;
@@ -67,4 +71,7 @@ struct stat {
   blkcnt_t  st_blocks;
 };
 
+__END_DECLS
+
+__POP_FC_STDLIB
 #endif
