@@ -2,7 +2,7 @@
 /*                                                                        */
 /*  This file is part of Frama-C.                                         */
 /*                                                                        */
-/*  Copyright (C) 2007-2015                                               */
+/*  Copyright (C) 2007-2018                                               */
 /*    CEA (Commissariat à l'énergie atomique et aux énergies              */
 /*         alternatives)                                                  */
 /*                                                                        */
@@ -22,9 +22,16 @@
 
 #ifndef __FC_DEFINE_WCHAR_T
 #define __FC_DEFINE_WCHAR_T
+#include "features.h"
+__PUSH_FC_STDLIB
+__BEGIN_DECLS
 #include "__fc_machdep.h"
 #if !defined(__cplusplus)
 /* wchar_t is a keyword in C++ and shall not be a typedef. */
 typedef __WCHAR_T wchar_t;
+#else
+typedef __WCHAR_T fc_wchar_t;
 #endif
+__END_DECLS
+__POP_FC_STDLIB
 #endif
