@@ -2,7 +2,7 @@
 /*                                                                        */
 /*  This file is part of Frama-C.                                         */
 /*                                                                        */
-/*  Copyright (C) 2007-2015                                               */
+/*  Copyright (C) 2007-2018                                               */
 /*    CEA (Commissariat à l'énergie atomique et aux énergies              */
 /*         alternatives)                                                  */
 /*                                                                        */
@@ -22,6 +22,10 @@
 
 #ifndef __FC_LOCALE
 #define __FC_LOCALE
+#include "features.h"
+__PUSH_FC_STDLIB
+
+__BEGIN_DECLS
 
 /* Structure giving information about numeric and monetary notation.  */
 struct lconv
@@ -87,28 +91,64 @@ struct lconv
 #include "__fc_define_null.h"
 #include "__fc_string_axiomatic.h"
 
-#define LC_ALL 0
-#define LC_COLLATE 1
-#define LC_CTYPE 2
-#define LC_MONETARY 3
-#define LC_NUMERIC 4
-#define LC_TIME 5
+#define LC_CTYPE           0
+#define LC_NUMERIC         1
+#define LC_TIME            2
+#define LC_COLLATE         3
+#define LC_MONETARY        4
+#define LC_MESSAGES        5
+#define LC_ALL             6
+#define LC_PAPER           7
+#define LC_NAME            8
+#define LC_ADDRESS         9
+#define LC_TELEPHONE      10
+#define LC_MEASUREMENT    11
+#define LC_IDENTIFICATION 12
+
+# define LC_CTYPE_MASK           (1 << LC_CTYPE)
+# define LC_NUMERIC_MASK         (1 << LC_NUMERIC)
+# define LC_TIME_MASK            (1 << LC_TIME)
+# define LC_COLLATE_MASK         (1 << LC_COLLATE)
+# define LC_MONETARY_MASK        (1 << LC_MONETARY)
+# define LC_MESSAGES_MASK        (1 << LC_MESSAGES)
+# define LC_PAPER_MASK           (1 << LC_PAPER)
+# define LC_NAME_MASK            (1 << LC_NAME)
+# define LC_ADDRESS_MASK         (1 << LC_ADDRESS)
+# define LC_TELEPHONE_MASK       (1 << LC_TELEPHONE)
+# define LC_MEASUREMENT_MASK     (1 << LC_MEASUREMENT)
+# define LC_IDENTIFICATION_MASK  (1 << LC_IDENTIFICATION)
+# define LC_ALL_MASK             (LC_CTYPE_MASK                 \
+                                  | LC_NUMERIC_MASK             \
+                                  | LC_TIME_MASK                \
+                                  | LC_COLLATE_MASK             \
+                                  | LC_MONETARY_MASK            \
+                                  | LC_MESSAGES_MASK            \
+                                  | LC_PAPER_MASK               \
+                                  | LC_NAME_MASK                \
+                                  | LC_ADDRESS_MASK             \
+                                  | LC_TELEPHONE_MASK           \
+                                  | LC_MEASUREMENT_MASK         \
+                                  | LC_IDENTIFICATION_MASK      \
+                                  )
 
 extern struct lconv* __frama_c_locale;
 extern char*__frama_c_locale_names[];
 
 /*@ 
-  requires locale == \null || valid_string(locale);
+  requires locale_null_or_valid_string: locale == \null || valid_read_string(locale);
   assigns __frama_c_locale \from category, locale[..];
   assigns \result \from __frama_c_locale,category, locale[..];
-  ensures \result==\null
-   || (\valid(\result) 
+  ensures result_null_or_locale_name: \result==\null || (\valid(\result) 
       && \exists ℤ i ; \result == __frama_c_locale_names[i]) ;
 */
 extern char *setlocale(int category, const char *locale);
 
 /*@ assigns \nothing;
-  ensures \result == __frama_c_locale;
+  ensures result_current_locale: \result == __frama_c_locale;
  */
 extern struct lconv *localeconv(void);
+
+__END_DECLS
+
+__POP_FC_STDLIB
 #endif
