@@ -2,7 +2,7 @@
 /*                                                                        */
 /*  This file is part of Frama-C.                                         */
 /*                                                                        */
-/*  Copyright (C) 2007-2015                                               */
+/*  Copyright (C) 2007-2018                                               */
 /*    CEA (Commissariat à l'énergie atomique et aux énergies              */
 /*         alternatives)                                                  */
 /*                                                                        */
@@ -23,6 +23,9 @@
 #ifndef __FC_UNISTD
 #define __FC_UNISTD
 
+#include "features.h"
+__PUSH_FC_STDLIB
+#include "__fc_string_axiomatic.h"
 #include "__fc_define_size_t.h"
 #include "__fc_define_null.h"
 #include "__fc_define_ssize_t.h"
@@ -34,6 +37,9 @@
 #include "__fc_select.h"
 
 #include <getopt.h>
+#include <limits.h>
+
+extern volatile int Frama_C_entropy_source;
 
 /* Values for the second argument to access.
    These may be OR'd together.  */
@@ -49,7 +55,17 @@
 
 #include "__fc_define_seek_macros.h"
 
+/* compatibility macros */
 
+#ifndef __FC_NO_MONOTONIC_CLOCK
+/* 0 indicates that the feature is supported at compile time, but might not
+   be supported at runtime. Frama-C can't make promises about the runtime
+   environment.
+*/
+#define _POSIX_MONOTONIC_CLOCK 0
+#endif
+
+__BEGIN_DECLS
 
 /* Values for the NAME argument to `pathconf' and `fpathconf'.  */
 enum
@@ -706,95 +722,257 @@ enum
   };
 
 
+// arbitrary number
+#define __FC_MAX_OPEN_FILES 1024
 
-int          access(const char *, int);
-unsigned int alarm(unsigned int);
-int          brk(void *);
-int          chdir(const char *path);
-int          chroot(const char *path);
-int          chown(const char *, uid_t, gid_t);
-int          close(int);
-size_t       confstr(int, char *, size_t);
-char        *crypt(const char *, const char *);
-char        *ctermid(char *);
-char        *cuserid(char *s);
-int          dup(int);
-int          dup2(int, int);
-void         encrypt(char[64], int);
-int          execl(const char *, const char *, ...);
-int          execle(const char *, const char *, ...);
-int          execlp(const char *, const char *, ...);
-int          execv(const char *, char *const []);
-int          execve(const char *, char *const [], char *const []);
-int          execvp(const char *, char *const []);
-void        _exit(int);
-int          fchown(int, uid_t, gid_t);
-int          fchdir(int);
-int          fdatasync(int);
-pid_t        fork(void);
-long int     fpathconf(int, int);
-int          fsync(int);
-int          ftruncate(int, off_t);
-char        *getcwd(char *, size_t);
-int          getdtablesize(void);
-gid_t        getegid(void);
-uid_t        geteuid(void);
-gid_t        getgid(void);
-int          getgroups(int, gid_t []);
-long         gethostid(void);
-int gethostname(char *, size_t);
-char        *getlogin(void);
-int          getlogin_r(char *, size_t);
-int          getpagesize(void);
-char        *getpass(const char *);
-pid_t        getpgid(pid_t);
-pid_t        getpgrp(void);
-pid_t        getpid(void);
-pid_t        getppid(void);
-pid_t        getsid(pid_t);
+// __fc_fds represents the state of open file descriptors.
+//@ ghost int __fc_fds[__FC_MAX_OPEN_FILES];
+// TODO: Model the state of some functions more precisely.
+// TODO: define __fc_fds as volatile.
+
+
+/*@ // missing: may assign to errno: EACCES, ELOOP, ENAMETOOLONG, ENOENT,
+    //                               ENOTDIR, EROFS, ETXTBSY
+    //                               (EINVAL prevented by precondition)
+    // missing: assigns \result \from 'filesystem, permissions'
+  requires valid_string_path: valid_read_string(path);
+  requires valid_amode: (amode & ~(R_OK | W_OK | X_OK)) == 0 ||
+                        amode == F_OK;
+  assigns \result \from indirect:path, indirect:path[0..], indirect:amode;
+  ensures result_ok_or_error: \result == 0 || \result == -1;
+*/
+extern int          access(const char *path, int amode);
+
+extern unsigned int alarm(unsigned int);
+extern int          brk(void *);
+extern int          chdir(const char *path);
+extern int          chroot(const char *path);
+extern int          chown(const char *, uid_t, gid_t);
+
+/*@
+  requires valid_fd: 0 <= fd < __FC_MAX_OPEN_FILES;
+  assigns __fc_fds[fd] \from fd, __fc_fds[fd];
+  assigns \result \from indirect:fd, indirect:__fc_fds[fd];
+  ensures result_ok_or_error: \result == 0 || \result == -1;
+*/
+extern int          close(int fd);
+extern size_t       confstr(int, char *, size_t);
+extern char        *crypt(const char *, const char *);
+extern char        *ctermid(char *);
+extern char        *cuserid(char *s);
+
+/*@ // missing: may assign errno EBADF, EMFILE
+  requires valid_fildes: 0 <= fildes < __FC_MAX_OPEN_FILES;
+  assigns __fc_fds[fildes..] \from fildes; // missing: \from 'filesystem'
+  assigns \result \from fildes;
+  ensures result_valid_fildes_or_error: \result == -1 ||
+    fildes <= \result < __FC_MAX_OPEN_FILES;
+*/
+extern int          dup(int fildes);
+
+/*@ // missing: may assign errno EBADF, EINTR, EIO
+  requires valid_fildes: 0 <= fildes < __FC_MAX_OPEN_FILES;
+  requires valid_fildes2: 0 <= fildes2 < __FC_MAX_OPEN_FILES;
+  assigns __fc_fds[fildes2] \from fildes, fildes2, __fc_fds[fildes2];
+  assigns \result \from fildes, fildes2, __fc_fds[fildes], __fc_fds[fildes2];
+  ensures result_fildes2_or_error: \result == fildes2 || \result == -1;
+*/
+extern int          dup2(int fildes, int fildes2);
+
+extern void         encrypt(char[64], int);
+
+/*@ requires valid_string_path: valid_read_string(path);
+    requires valid_string_arg: valid_read_string(arg);
+    assigns \result \from path[0..], arg[0..];
+*/
+extern int          execl(const char *path, const char *arg, ...);
+/*@ requires valid_string_path: valid_read_string(path);
+    requires valid_string_arg: valid_read_string(arg);
+    assigns \result \from path[0..], arg[0..];
+*/
+extern int          execle(const char *path, const char *arg, ...);
+/*@ requires valid_string_path: valid_read_string(path);
+    requires valid_string_arg: valid_read_string(arg);
+    assigns \result \from path[0..], arg[0..];
+*/
+extern int          execlp(const char *path, const char *arg, ...);
+/*@ requires valid_string_path: valid_read_string(path);
+    requires valid_string_argv0: valid_read_string(argv[0]);
+    assigns \result \from path[0..], argv[0..];
+*/
+extern int          execv(const char *path, char *const argv[]);
+/*@ requires valid_path: valid_read_string(path);
+    requires valid_argv0: valid_read_string(argv[0]);
+    assigns \result \from path[0..], argv[0..];
+*/
+extern int          execve(const char *path, char *const argv[], char *const env[]);
+/*@ requires valid_string_path: valid_read_string(path);
+    requires valid_string_argv0: valid_read_string(argv[0]);
+    assigns \result \from path[0..], argv[0..];
+*/
+extern int          execvp(const char *path, char *const argv[]);
+
+extern void         _exit(int) __attribute__ ((__noreturn__));
+extern int          fchown(int, uid_t, gid_t);
+extern int          fchdir(int);
+extern int          fdatasync(int);
+
+/*@ // missing: assigns \result \from 'other processes, internal state'
+    // missing: may assign errno EAGAIN, ENOMEM
+  assigns \result \from \nothing;
+  ensures result_ok_child_or_error: \result == 0 || \result > 0 ||
+    \result == -1;
+*/
+extern pid_t        fork(void);
+
+extern long int     fpathconf(int, int);
+extern int          fsync(int);
+extern int          ftruncate(int, off_t);
+extern char        *getcwd(char *, size_t);
+extern int          getdtablesize(void);
+extern gid_t        getegid(void);
+extern uid_t        geteuid(void);
+extern gid_t        getgid(void);
+extern int          getgroups(int, gid_t []);
+extern long         gethostid(void);
+
+extern volatile char __fc_hostname[HOST_NAME_MAX];
+
+/*@
+  requires name_has_room: \valid(name + (0 .. len-1));
+  assigns \result, name[0 .. len-1]
+      \from indirect:__fc_hostname[0 .. len], indirect:len;
+  ensures result_ok_or_error: \result == 0 || \result == -1;
+*/
+extern int gethostname(char *name, size_t len);
+
+// Non-POSIX
+/*@
+  requires name_valid_string: valid_read_nstring(name, len);
+  requires bounded_len: len <= HOST_NAME_MAX;
+  assigns __fc_hostname[0 .. len] \from name[0 .. len-1], indirect:len;
+  assigns \result \from indirect:__fc_hostname[0 .. len];
+  ensures result_ok_or_error: \result == 0 || \result == -1;
+*/
+extern int sethostname(const char *name, size_t len);
+
+extern char        *getlogin(void);
+extern int          getlogin_r(char *, size_t);
+extern int          getpagesize(void);
+extern char        *getpass(const char *);
+extern pid_t        getpgid(pid_t);
+extern pid_t        getpgrp(void);
+extern pid_t        getpid(void);
+extern pid_t        getppid(void);
+extern pid_t        getsid(pid_t);
 /*@ assigns \result \from \nothing; */
-uid_t        getuid(void);
-char        *getwd(char *);
-int          isatty(int);
-int          lchown(const char *, uid_t, gid_t);
-int          link(const char *, const char *);
-int          lockf(int, int, off_t);
-off_t        lseek(int, off_t, int);
-int          nice(int);
-long int     pathconf(const char *, int);
-int          pause(void);
-int          pipe(int [2]);
-ssize_t      pread(int, void *, size_t, off_t);
-int          pthread_atfork(void (*)(void), void (*)(void),
+extern uid_t        getuid(void);
+extern char        *getwd(char *);
+extern int          isatty(int);
+extern int          lchown(const char *, uid_t, gid_t);
+extern int          link(const char *, const char *);
+extern int          lockf(int, int, off_t);
+extern off_t        lseek(int, off_t, int);
+extern int          nice(int);
+extern long int     pathconf(const char *, int);
+extern int          pause(void);
+
+/*@
+  assigns pipefd[0..1] \from indirect:__fc_fds[0..];
+  assigns \result \from indirect:__fc_fds[0..];
+  ensures initialization:pipefd: \initialized(&pipefd[0..1]);
+  ensures valid_fd0: 0 <= pipefd[0] < __FC_MAX_OPEN_FILES;
+  ensures valid_fd1: 0 <= pipefd[1] < __FC_MAX_OPEN_FILES;
+  ensures result_ok_or_error: \result == 0 || \result == -1;
+ */
+extern int          pipe(int pipefd[2]);
+
+extern ssize_t      pread(int, void *, size_t, off_t);
+extern int          pthread_atfork(void (*)(void), void (*)(void),
                  void(*)(void));
-ssize_t      pwrite(int, const void *, size_t, off_t);
-ssize_t      read(int, void *, size_t);
-int          readlink(const char *, char *, size_t);
-int          rmdir(const char *);
-void        *sbrk(intptr_t);
-int setegid(gid_t gid);
-int seteuid(uid_t uid);
-int          setgid(gid_t);
-int          setpgid(pid_t, pid_t);
-pid_t        setpgrp(void);
-int          setregid(gid_t, gid_t);
-int          setreuid(uid_t, uid_t);
-pid_t        setsid(void);
-int          setuid(uid_t uid);
-unsigned int sleep(unsigned int);
-void         swab(const void *, void *, ssize_t);
-int          symlink(const char *, const char *);
-void         sync(void);
-long int     sysconf(int);
-pid_t        tcgetpgrp(int);
-int          tcsetpgrp(int, pid_t);
-int          truncate(const char *, off_t);
-char        *ttyname(int);
-int          ttyname_r(int, char *, size_t);
-useconds_t   ualarm(useconds_t, useconds_t);
-int          unlink(const char *);
-int          usleep(useconds_t);
-pid_t        vfork(void);
-ssize_t      write(int, const void *, size_t);
+extern ssize_t      pwrite(int, const void *, size_t, off_t);
+
+/*@
+  requires valid_fd: 0 <= fd < __FC_MAX_OPEN_FILES;
+  requires buf_has_room: \valid((char *)buf+(0..count-1));
+  assigns __fc_fds[fd] \from __fc_fds[fd];
+  assigns \result, *((char *)buf+(0..count-1))
+          \from indirect:__fc_fds[fd], indirect:count;
+  ensures  result_error_or_read_length: 0 <= \result <= count || \result == -1;
+  ensures  initialization:buf: \initialized(((char*)buf)+(0..\result-1));
+*/
+extern ssize_t      read(int fd, void *buf, size_t count);
+
+extern int          readlink(const char *, char *, size_t);
+extern int          rmdir(const char *);
+extern void        *sbrk(intptr_t);
+extern int setegid(gid_t gid);
+extern int seteuid(uid_t uid);
+extern int          setgid(gid_t);
+extern int          setpgid(pid_t, pid_t);
+extern pid_t        setpgrp(void);
+extern int          setregid(gid_t, gid_t);
+extern int          setreuid(uid_t, uid_t);
+
+/*@ // missing: may assign errno to EPERM
+    // missing: assigns 'processes' \from 'processes'
+  assigns \result \from \nothing;
+  ensures result_new_proc_group_or_error: \result >= 0 || \result == -1;
+*/
+extern pid_t        setsid(void);
+
+extern int          setuid(uid_t uid);
+extern unsigned int sleep(unsigned int);
+extern void         swab(const void *, void *, ssize_t);
+extern int          symlink(const char *, const char *);
+
+/*@ //missing: assigns 'filesystem' \from 'filesystem'
+  assigns \nothing;
+*/
+extern void         sync(void);
+
+/*@ //missing: assigns 'filesystem', \result \from 'filesystem'
+    //missing: may set errno to EINVAL
+  assigns \result \from indirect:name;
+*/
+extern long int     sysconf(int name);
+
+extern pid_t        tcgetpgrp(int);
+extern int          tcsetpgrp(int, pid_t);
+extern int          truncate(const char *, off_t);
+extern char        *ttyname(int);
+extern int          ttyname_r(int, char *, size_t);
+extern useconds_t   ualarm(useconds_t, useconds_t);
+extern int          unlink(const char *);
+
+// usleep is not POSIX anymore since 200809
+#if (_XOPEN_SOURCE >= 500) && ! (_POSIX_C_SOURCE >= 200809L) \
+  || /* Glibc since 2.19: */ defined _DEFAULT_SOURCE \
+  || /* Glibc versions <= 2.19: */ defined _BSD_SOURCE
+/*@
+  assigns \result \from indirect:usec, indirect:Frama_C_entropy_source;
+  assigns Frama_C_entropy_source \from Frama_C_entropy_source;
+  ensures result_ok_or_error: \result == 0 || \result == -1;
+ */
+extern int          usleep(useconds_t usec);
 
 #endif
+
+extern pid_t        vfork(void);
+
+/*@
+  requires valid_fd: 0 <= fd < __FC_MAX_OPEN_FILES;
+  requires buf_has_room: \valid_read(((char *)buf)+(0..count-1));
+  assigns __fc_fds[fd] \from indirect:fd, indirect:count, __fc_fds[fd];
+  assigns \result \from indirect:fd, indirect:count, indirect:__fc_fds[fd];
+  ensures result_error_or_written_bytes: \result == -1 || 0 <= \result <= count;
+*/
+extern ssize_t      write(int fd, const void *buf, size_t count);
+
+// setgroups() is not POSIX
+extern int setgroups(size_t size, const gid_t *list);
+
+__END_DECLS
+
+__POP_FC_STDLIB
+#endif
